@@ -5,11 +5,7 @@ Copyright (C) 2014 Ömer Özak
 This program defines functions that are useful for working with GIS data
 Usage:
 
-import gisrastertools
-
-or
-
-from gisrastertools import *
+import georasters as gr
 
 ======================================================
 Author:  Ömer Özak, 2013--2014 (ozak at smu.edu)
@@ -40,156 +36,156 @@ import pandas as pd
 
 # Function to read the original file's projection:
 def get_geo_info(FileName):
-	''' Gets information from a Raster data set
-	'''
-	SourceDS = gdal.Open(FileName, GA_ReadOnly)
-	NDV = SourceDS.GetRasterBand(1).GetNoDataValue()
-	xsize = SourceDS.RasterXSize
-	ysize = SourceDS.RasterYSize
-	GeoT = SourceDS.GetGeoTransform()
-	Projection = osr.SpatialReference()
-	Projection.ImportFromWkt(SourceDS.GetProjectionRef())
-	DataType = SourceDS.GetRasterBand(1).DataType
-	DataType = gdal.GetDataTypeName(DataType)
-	return NDV, xsize, ysize, GeoT, Projection, DataType
+    ''' Gets information from a Raster data set
+    '''
+    SourceDS = gdal.Open(FileName, GA_ReadOnly)
+    NDV = SourceDS.GetRasterBand(1).GetNoDataValue()
+    xsize = SourceDS.RasterXSize
+    ysize = SourceDS.RasterYSize
+    GeoT = SourceDS.GetGeoTransform()
+    Projection = osr.SpatialReference()
+    Projection.ImportFromWkt(SourceDS.GetProjectionRef())
+    DataType = SourceDS.GetRasterBand(1).DataType
+    DataType = gdal.GetDataTypeName(DataType)
+    return NDV, xsize, ysize, GeoT, Projection, DataType
 
 # Function to map location in pixel of raster array
 def map_pixel(point_x, point_y, cellx, celly, xmin, ymax):
-	'''
-	Usage: map_pixel(xcoord, ycoord, x_cell_size, y_cell_size, xmin, ymax)
-	where: 
-			xmin is leftmost X coordinate in system
-			ymax is topmost Y coordinate in system
-	Example:
-			raster = HMISea.tif'
-			NDV, xsize, ysize, GeoT, Projection, DataType = GetGeoInfo(raster)
-			col, row = map_pixel(x,y,GeoT[1],GeoT[-1], GeoT[0],GeoT[3])
-	'''
-	point_x=np.array(point_x)
-	point_y=np.array(point_y)
-	col = np.around((point_x - xmin) / cellx).astype(int)
-	row = np.around((point_y - ymax) / celly).astype(int)
-	return row,col
+    '''
+    Usage: map_pixel(xcoord, ycoord, x_cell_size, y_cell_size, xmin, ymax)
+    where: 
+            xmin is leftmost X coordinate in system
+            ymax is topmost Y coordinate in system
+    Example:
+            raster = HMISea.tif'
+            NDV, xsize, ysize, GeoT, Projection, DataType = GetGeoInfo(raster)
+            col, row = map_pixel(x,y,GeoT[1],GeoT[-1], GeoT[0],GeoT[3])
+    '''
+    point_x=np.array(point_x)
+    point_y=np.array(point_y)
+    col = np.around((point_x - xmin) / cellx).astype(int)
+    row = np.around((point_y - ymax) / celly).astype(int)
+    return row,col
 
 # Aggregate raster to higher resolution using sums
 def aggregate(raster,NDV,block_size):
-	'''
-	Aggregate raster to smaller resolution, by adding cells.
-	Usage:
-			aggregate(raster,NDV,block_size)
-	where
-			raster is a Numpy array created by importing the raster (e.g. GeoTiff)
-			NDV is the NoData Value for the raster (can be read using the GetGeoInfo function)
-			block_size is a duple of factors by which the raster will be shrinked
-	Example:
-			raster = HMISea.tif'
-			NDV, xsize, ysize, GeoT, Projection, DataType = GetGeoInfo(raster)
-			costs = load_tiff(raster)
-			costs2=aggregate(costs,NDV,(10,10))
-	'''
-	raster2=np.where(raster==NDV,0,raster)
-	raster3=block_reduce(raster2,block_size,func=np.sum)
-	raster2=np.where(raster==NDV,NDV,0)
-	raster4=block_reduce(raster2,block_size,func=np.sum)
-	raster2=np.where(raster4<0,NDV,raster3)
-	return raster2
+    '''
+    Aggregate raster to smaller resolution, by adding cells.
+    Usage:
+            aggregate(raster,NDV,block_size)
+    where
+            raster is a Numpy array created by importing the raster (e.g. GeoTiff)
+            NDV is the NoData Value for the raster (can be read using the GetGeoInfo function)
+            block_size is a duple of factors by which the raster will be shrinked
+    Example:
+            raster = HMISea.tif'
+            NDV, xsize, ysize, GeoT, Projection, DataType = GetGeoInfo(raster)
+            costs = load_tiff(raster)
+            costs2=aggregate(costs,NDV,(10,10))
+    '''
+    raster2=np.where(raster==NDV,0,raster)
+    raster3=block_reduce(raster2,block_size,func=np.sum)
+    raster2=np.where(raster==NDV,NDV,0)
+    raster4=block_reduce(raster2,block_size,func=np.sum)
+    raster2=np.where(raster4<0,NDV,raster3)
+    return raster2
 
 # Function to write a new file.
 def create_geotiff(Name, Array, driver, NDV, xsize, ysize, GeoT, Projection, DataType):
-	'''
-	Creates new GeoTiff from array
-	'''
-	if type(DataType)!=np.int:
-		if DataType.startswith('gdal.GDT_')==False:
-			DataType=eval('gdal.GDT_'+DataType)
-	NewFileName = Name+'.tif'
-	# Set nans to the original No Data Value
-	Array[np.isnan(Array)] = NDV
-	# Set up the dataset
-	DataSet = driver.Create( NewFileName, xsize, ysize, 1, DataType)
-	# the '1' is for band 1.
-	DataSet.SetGeoTransform(GeoT)
-	DataSet.SetProjection( Projection.ExportToWkt() )
-	# Write the array
-	DataSet.GetRasterBand(1).WriteArray( Array )
-	DataSet.GetRasterBand(1).SetNoDataValue(NDV)
-	return NewFileName
+    '''
+    Creates new GeoTiff from array
+    '''
+    if type(DataType)!=np.int:
+        if DataType.startswith('gdal.GDT_')==False:
+            DataType=eval('gdal.GDT_'+DataType)
+    NewFileName = Name+'.tif'
+    # Set nans to the original No Data Value
+    Array[np.isnan(Array)] = NDV
+    # Set up the dataset
+    DataSet = driver.Create( NewFileName, xsize, ysize, 1, DataType)
+    # the '1' is for band 1.
+    DataSet.SetGeoTransform(GeoT)
+    DataSet.SetProjection( Projection.ExportToWkt() )
+    # Write the array
+    DataSet.GetRasterBand(1).WriteArray( Array )
+    DataSet.GetRasterBand(1).SetNoDataValue(NDV)
+    return NewFileName
 
 # Function to aggregate and align rasters
 def align_rasters(raster,alignraster,how=np.mean,cxsize=None,cysize=None,masked=False):
-	'''
-	Align two rasters so that data overlaps by geographical location
-	Usage: (alignedraster_o, alignedraster_a, GeoT_a) = AlignRasters(raster, alignraster, how=np.mean)
-	where 
-		raster: string with location of raster to be aligned
-		alignraster: string with location of raster to which raster will be aligned
-		how: function used to aggregate cells (if the rasters have different sizes)
-	It is assumed that both rasters have the same size
-	'''
-	NDV1, xsize1, ysize1, GeoT1, Projection1, DataType1=GetGeoInfo(raster)
-	NDV2, xsize2, ysize2, GeoT2, Projection2, DataType2=GetGeoInfo(alignraster)
-	if Projection1.ExportToMICoordSys()==Projection2.ExportToMICoordSys():
-		blocksize=(np.round(GeoT2[1]/GeoT1[1]),np.round(GeoT2[-1]/GeoT1[-1]))
-		mraster=gdalnumeric.LoadFile(raster)
-		mraster=np.ma.masked_array(mraster, mask=mraster==NDV1, fill_value=NDV1)
-		mmin=mraster.min()
-		mraster=block_reduce(mraster,blocksize,func=how)
-		araster=gdalnumeric.LoadFile(alignraster)
-		araster=np.ma.masked_array(araster, mask=araster==NDV2, fill_value=NDV2)
-		amin=araster.min()
-		if GeoT1[0]<=GeoT2[0]:
-			row3,mcol=map_pixel(GeoT2[0], GeoT2[3], GeoT1[1] *blocksize[0],GeoT1[-1]*blocksize[1], GeoT1[0], GeoT1[3])
-			acol=0
-		else:
-			row3,acol=map_pixel(GeoT1[0], GeoT1[3], GeoT2[1],GeoT2[-1], GeoT2[0], GeoT2[3])
-			mcol=0
-		if GeoT1[3]<=GeoT2[3]:
-			arow,col3=map_pixel(GeoT1[0], GeoT1[3], GeoT2[1],GeoT2[-1], GeoT2[0], GeoT2[3])
-			mrow=0
-		else:
-			mrow,col3=map_pixel(GeoT2[0], GeoT2[3], GeoT1[1] *blocksize[0],GeoT1[-1]*blocksize[1], GeoT1[0], GeoT1[3])
-			arow=0
-		'''
-		col3,row3=map_pixel(GeoT1[0], GeoT1[3], GeoT2[1],GeoT2[-1], GeoT2[0], GeoT2[3])
-		col3=max(0,col3)
-		row3=max(0,row3)
-		araster=araster[row3:,col3:]
-		col3,row3=map_pixel(GeoT2[0], GeoT2[3], GeoT1[1] *blocksize[0],GeoT1[-1]*blocksize[1], GeoT1[0], GeoT1[3])
-		col3=max(0,abs(col3))
-		row3=max(0,np.abs(row3))
-		mraster=mraster[row3:,col3:]
-		'''
-		mraster=mraster[mrow:,mcol:]
-		araster=araster[arow:,acol:]
-		if cxsize and cysize:
-			araster=araster[:cysize,:cxsize]
-			mraster=mraster[:cysize,:cxsize]
-		else:
-			rows = min(araster.shape[0],mraster.shape[0])
-			cols = min(araster.shape[1],mraster.shape[1])
-			araster=araster[:rows,:cols]
-			mraster=mraster[:rows,:cols]
-		#mraster=mraster[row3:rows+row3,col3:cols+col3]
-		if masked:
-			mraster=np.ma.masked_array(mraster,mask=mraster<mmin, fill_value=NDV1)
-			araster=np.ma.masked_array(araster,mask=araster<amin, fill_value=NDV2)
-		GeoT=(max(GeoT1[0],GeoT2[0]), GeoT1[1]*blocksize[0], GeoT1[2], min(GeoT1[3],GeoT2[3]), GeoT1[4] ,GeoT1[-1]*blocksize[1])
-		return (mraster,araster,GeoT)
-	else:
-		print("Rasters need to be in same projection")
-		return (-1,-1,-1)
+    '''
+    Align two rasters so that data overlaps by geographical location
+    Usage: (alignedraster_o, alignedraster_a, GeoT_a) = AlignRasters(raster, alignraster, how=np.mean)
+    where 
+        raster: string with location of raster to be aligned
+        alignraster: string with location of raster to which raster will be aligned
+        how: function used to aggregate cells (if the rasters have different sizes)
+    It is assumed that both rasters have the same size
+    '''
+    NDV1, xsize1, ysize1, GeoT1, Projection1, DataType1=GetGeoInfo(raster)
+    NDV2, xsize2, ysize2, GeoT2, Projection2, DataType2=GetGeoInfo(alignraster)
+    if Projection1.ExportToMICoordSys()==Projection2.ExportToMICoordSys():
+        blocksize=(np.round(GeoT2[1]/GeoT1[1]),np.round(GeoT2[-1]/GeoT1[-1]))
+        mraster=gdalnumeric.LoadFile(raster)
+        mraster=np.ma.masked_array(mraster, mask=mraster==NDV1, fill_value=NDV1)
+        mmin=mraster.min()
+        mraster=block_reduce(mraster,blocksize,func=how)
+        araster=gdalnumeric.LoadFile(alignraster)
+        araster=np.ma.masked_array(araster, mask=araster==NDV2, fill_value=NDV2)
+        amin=araster.min()
+        if GeoT1[0]<=GeoT2[0]:
+            row3,mcol=map_pixel(GeoT2[0], GeoT2[3], GeoT1[1] *blocksize[0],GeoT1[-1]*blocksize[1], GeoT1[0], GeoT1[3])
+            acol=0
+        else:
+            row3,acol=map_pixel(GeoT1[0], GeoT1[3], GeoT2[1],GeoT2[-1], GeoT2[0], GeoT2[3])
+            mcol=0
+        if GeoT1[3]<=GeoT2[3]:
+            arow,col3=map_pixel(GeoT1[0], GeoT1[3], GeoT2[1],GeoT2[-1], GeoT2[0], GeoT2[3])
+            mrow=0
+        else:
+            mrow,col3=map_pixel(GeoT2[0], GeoT2[3], GeoT1[1] *blocksize[0],GeoT1[-1]*blocksize[1], GeoT1[0], GeoT1[3])
+            arow=0
+        '''
+        col3,row3=map_pixel(GeoT1[0], GeoT1[3], GeoT2[1],GeoT2[-1], GeoT2[0], GeoT2[3])
+        col3=max(0,col3)
+        row3=max(0,row3)
+        araster=araster[row3:,col3:]
+        col3,row3=map_pixel(GeoT2[0], GeoT2[3], GeoT1[1] *blocksize[0],GeoT1[-1]*blocksize[1], GeoT1[0], GeoT1[3])
+        col3=max(0,abs(col3))
+        row3=max(0,np.abs(row3))
+        mraster=mraster[row3:,col3:]
+        '''
+        mraster=mraster[mrow:,mcol:]
+        araster=araster[arow:,acol:]
+        if cxsize and cysize:
+            araster=araster[:cysize,:cxsize]
+            mraster=mraster[:cysize,:cxsize]
+        else:
+            rows = min(araster.shape[0],mraster.shape[0])
+            cols = min(araster.shape[1],mraster.shape[1])
+            araster=araster[:rows,:cols]
+            mraster=mraster[:rows,:cols]
+        #mraster=mraster[row3:rows+row3,col3:cols+col3]
+        if masked:
+            mraster=np.ma.masked_array(mraster,mask=mraster<mmin, fill_value=NDV1)
+            araster=np.ma.masked_array(araster,mask=araster<amin, fill_value=NDV2)
+        GeoT=(max(GeoT1[0],GeoT2[0]), GeoT1[1]*blocksize[0], GeoT1[2], min(GeoT1[3],GeoT2[3]), GeoT1[4] ,GeoT1[-1]*blocksize[1])
+        return (mraster,araster,GeoT)
+    else:
+        print("Rasters need to be in same projection")
+        return (-1,-1,-1)
 
 # Load GeoTif raster data
 def load_tiff(file):
-	"""
-	Load a GeoTiff raster keeping NDV values using a masked array
-	Usage:
-			data=LoadTiffRaster(file)
-	"""
-	NDV, xsize, ysize, GeoT, Projection, DataType=GetGeoInfo(file)
-	data=gdalnumeric.LoadFile(file)
-	data=np.ma.masked_array(data, mask=data==NDV,fill_value=-np.inf)
-	return data
+    """
+    Load a GeoTiff raster keeping NDV values using a masked array
+    Usage:
+            data=LoadTiffRaster(file)
+    """
+    NDV, xsize, ysize, GeoT, Projection, DataType=GetGeoInfo(file)
+    data=gdalnumeric.LoadFile(file)
+    data=np.ma.masked_array(data, mask=data==NDV,fill_value=-np.inf)
+    return data
 
 class RasterGeoTError(Exception):
     pass
@@ -593,6 +589,19 @@ class GeoRaster():
         return GeoRaster(self.raster[max(row-row2, 0):min(row+row2+1, self.shape[0]), \
                         max(col-col2, 0):min(col+col2+1, self.shape[1])], self.geot, nodata_value = self.nodata_value)
 
+    # Align GeoRasters
+    def align(self,alignraster,how=np.mean,cxsize=None,cysize=None):
+        '''
+        Align two rasters so that data overlaps by geographical location
+        Usage: (alignedraster_o, alignedraster_a) = AlignRasters(raster, alignraster, how=np.mean)
+        where 
+            raster: string with location of raster to be aligned
+            alignraster: string with location of raster to which raster will be aligned
+            how: function used to aggregate cells (if the rasters have different sizes)
+        It is assumed that both rasters have the same size
+        '''
+        return align_georasters(self,alignraster,how=how,cxsize=cxsize,cysize=cysize)
+
 # Union of rasters
 def union(rasters):
     """
@@ -613,7 +622,7 @@ def union(rasters):
             datatype=rasters[0].datatype
         else:
             datatype = None
-        projection = rasters[0].projection.ExportToProj4()
+        projection = rasters[0].projection
         lonmin = min([i.xmin for i in rasters])
         lonmax = max([i.xmax for i in rasters])
         latmin = min([i.ymin for i in rasters])
@@ -661,3 +670,59 @@ NP2GDAL_CONVERSION = {
   "complex64": 10,
   "complex128": 11,
 }
+
+# Align GeoRasters
+def align_georasters(raster,alignraster,how=np.mean,cxsize=None,cysize=None):
+    '''
+    Align two rasters so that data overlaps by geographical location
+    Usage: (alignedraster_o, alignedraster_a) = AlignRasters(raster, alignraster, how=np.mean)
+    where 
+        raster: string with location of raster to be aligned
+        alignraster: string with location of raster to which raster will be aligned
+        how: function used to aggregate cells (if the rasters have different sizes)
+    It is assumed that both rasters have the same size
+    '''
+    (NDV1, xsize1, ysize1, GeoT1, Projection1, DataType1)=(raster.nodata_value, raster.shape[1], raster.shape[0], raster.geot, raster.projection, raster.datatype)
+    (NDV2, xsize2, ysize2, GeoT2, Projection2, DataType2)=(alignraster.nodata_value, alignraster.shape[1], alignraster.shape[0], alignraster.geot, alignraster.projection, alignraster.datatype)
+    if Projection1.ExportToMICoordSys()==Projection2.ExportToMICoordSys():
+        blocksize=(np.round(max(GeoT2[1]/GeoT1[1],1)),np.round(max(GeoT2[-1]/GeoT1[-1],1)))
+        mraster=raster.raster
+        mmin=mraster.min()
+        if block_reduce!=(1,1):
+            mraster=block_reduce(mraster,blocksize,func=how)
+        blocksize=(np.round(max(GeoT1[1]/GeoT2[1],1)),np.round(max(GeoT1[-1]/GeoT2[-1],1)))
+        araster=alignraster.raster
+        amin=araster.min()
+        if block_reduce!=(1,1):
+            araster=block_reduce(araster,blocksize,func=how)
+        if GeoT1[0]<=GeoT2[0]:
+            row3,mcol=map_pixel(GeoT2[0], GeoT2[3], GeoT1[1] *blocksize[0],GeoT1[-1]*blocksize[1], GeoT1[0], GeoT1[3])
+            acol=0
+        else:
+            row3,acol=map_pixel(GeoT1[0], GeoT1[3], GeoT2[1],GeoT2[-1], GeoT2[0], GeoT2[3])
+            mcol=0
+        if GeoT1[3]<=GeoT2[3]:
+            arow,col3=map_pixel(GeoT1[0], GeoT1[3], GeoT2[1],GeoT2[-1], GeoT2[0], GeoT2[3])
+            mrow=0
+        else:
+            mrow,col3=map_pixel(GeoT2[0], GeoT2[3], GeoT1[1] *blocksize[0],GeoT1[-1]*blocksize[1], GeoT1[0], GeoT1[3])
+            arow=0
+        mraster=mraster[mrow:,mcol:]
+        araster=araster[arow:,acol:]
+        if cxsize and cysize:
+            araster=araster[:cysize,:cxsize]
+            mraster=mraster[:cysize,:cxsize]
+        else:
+            rows = min(araster.shape[0],mraster.shape[0])
+            cols = min(araster.shape[1],mraster.shape[1])
+            araster=araster[:rows,:cols]
+            mraster=mraster[:rows,:cols]
+        mraster=np.ma.masked_array(mraster,mask=mraster<mmin, fill_value=NDV1)
+        araster=np.ma.masked_array(araster,mask=araster<amin, fill_value=NDV2)
+        GeoT=(max(GeoT1[0],GeoT2[0]), GeoT1[1]*blocksize[0], GeoT1[2], min(GeoT1[3],GeoT2[3]), GeoT1[4] ,GeoT1[-1]*blocksize[1])
+        mraster=GeoRaster(mraster, GeoT, projection=Projection1, nodata_value=NDV1, datatype=DataType1)
+        araster=GeoRaster(araster, GeoT, projection=Projection2, nodata_value=NDV2, datatype=DataType2)
+        return (mraster,araster)
+    else:
+        print("Rasters need to be in same projection")
+        return (-1,-1)
